==> 1.7.8&file=out_yasapi-1.7.8-src.tar.gz: out_yasapi-1.7.8/libya/ya_player_stub.c <==
/*
 * ya_player_stub.c
 * Copyright (C) 2015-2016 Peter Belkner <pbelkner@snafu.de>
 *
 * This file is part of libya.
 *
 * libya is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libya is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libya.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <ya.h>
#include <avrt.h>

static PlayerStub stub;

PlayerStub *PlayerStubCreate(const IPlayer *lpVtbl, Player *pPlayer)
{
  PlayerStub *pStub=&stub;

  /////////////////////////////////////////////////////////////////////////////
  if (pStub->lpVtbl) {
    DMESSAGE("vtbl already in use");
    goto vtbl;
  }

  pStub->lpVtbl=lpVtbl;

  if (pStub->pPlayer) {
    DMESSAGE("player already in use");
    goto player;
  }

  pStub->pPlayer=pPlayer;

#if defined (YA_DEBUG) // {
  /////////////////////////////////////////////////////////////////////////////
  TraceFixIdcs(&trace,lpVtbl->pTraceIdcs);
#endif // }

  /////////////////////////////////////////////////////////////////////////////
  if (QueueCreate(&pStub->queue)<0) {
    DMESSAGE("creating queue");
    goto queue;
  }

  DPUTS(0,"  queue created\n");

  /////////////////////////////////////////////////////////////////////////////
  if (StoreCreate(&pStub->store)<0) {
    DMESSAGE("creating store");
    goto store;
  }

  DPUTS(0,"  store created\n");

  // create the thread ////////////////////////////////////////////////////////
  pStub->hThread=CreateThread(
    NULL,               // _In_opt_   LPSECURITY_ATTRIBUTES lpThreadAttributes,
    0,                  // _In_       SIZE_T dwStackSize,
    PlayerStubThread,   // _In_       LPTHREAD_START_ROUTINE lpStartAddress,
    pStub,              // _In_opt_   LPVOID lpParameter,
    0,                  // _In_       DWORD dwCreationFlags,
    NULL                // _Out_opt_  LPDWORD lpThreadId
  );

  if (NULL==pStub->hThread) {
    DMESSAGE("creating thread");
    goto thread;
  }

  DPUTS(0,"  thread created\n");

  /////////////////////////////////////////////////////////////////////////////
  if (PlayerStubSend(pStub,0,lpVtbl->RunProc)<0) {
    DMESSAGE("running");
    goto run;
  }

  DPUTS(0,"  run request send\n");

  /////////////////////////////////////////////////////////////////////////////
  return pStub;
run:
  CloseHandle(pStub->hThread);
thread:
  StoreDestroy(&pStub->store);
store:
  QueueDestroy(&pStub->queue);
queue:
player:
vtbl:
  return NULL;
}

void PlayerStubDestroy(PlayerStub *pStub)
{
  PlayerStubSend(pStub,1,pStub->lpVtbl->StopProc);
  WaitForSingleObject(pStub->hThread,INFINITE);
  CloseHandle(pStub->hThread);
  StoreDestroy(&pStub->store);
  QueueDestroy(&pStub->queue);
  pStub->pPlayer=NULL;
  pStub->lpVtbl=NULL;
}

DWORD WINAPI PlayerStubThread(LPVOID lpParameter)
{
  PlayerStub *pStub=lpParameter;
  const IPlayer *lpVtbl=pStub->lpVtbl;
  Player *pPlayer=pStub->pPlayer;
  Queue *pQueue=&pStub->queue;
  PlayerEventProc *EventProc=lpVtbl->EventProc;
  int exit=0;
  DWORD dwTaskIndex=0;
  HANDLE hTask=AvSetMmThreadCharacteristicsW(L"Pro Audio",&dwTaskIndex);
  HANDLE hEvent;
  Request *pRequest;
  int state;
#if defined (YA_THREAD_CO_INITIALIZE) // {
  HRESULT hr;

  hr=CoInitializeEx(
    NULL,                     // _In_opt_ LPVOID pvReserved
	  COINIT_MULTITHREADED		  // _In_     DWORD  dwCoInit
  );

  if (FAILED(hr)) {
    DERROR(S_FALSE,hr);
    DERROR(RPC_E_CHANGED_MODE,hr);
    DMESSAGE("initializing com failed");
    goto coinit;
  }
#endif // }

  while (!exit) {
    hEvent=lpVtbl->GetEvent(pPlayer);
    pRequest=QueueLockRead(pQueue,hEvent,EventProc,pPlayer);
    state=pRequest->pPlayerProc(pPlayer,pRequest);
    exit=pRequest->exit;

    if (NULL!=pRequest->pResult)
      pRequest->pResult->state=state;

    QueueUnlockRead(pQueue,pRequest->pResult);
  }

#if defined (YA_THREAD_CO_INITIALIZE) // {
  CoUninitialize();
coinit:
#endif // }

  if (hTask)
    AvRevertMmThreadCharacteristics(hTask);

  return 0;
}

///////////////////////////////////////////////////////////////////////////////
int64_t PlayerStubSendV(PlayerStub *pStub, int exit, PlayerProc *pPlayerProc,
    va_list ap)
{
  Store *pStore=&pStub->store;
  Queue *pQueue=&pStub->queue;
  Result *pResult;
  Request *pRequest;
  int64_t state;

  pResult=StoreGet(pStore);
  pRequest=QueueLockWrite(pQueue,pResult,exit,pPlayerProc,ap);
  QueueUnlockWrite(pQueue,pResult);
    state=pResult->state;
  StorePut(pStore,pResult);

  return state;
}

int64_t PlayerStubSend(PlayerStub *pStub, int exit, PlayerProc *pPlayerProc,
    ...)
{
  va_list ap;
  int64_t state;

  va_start(ap,pPlayerProc);
  state=PlayerStubSendV(pStub,exit,pPlayerProc,ap);
  va_end(ap);

  return state;
}

void PlayerStubPost(PlayerStub *pStub, int exit, PlayerProc *pPlayerProc)
{
  Queue *pQueue=&pStub->queue;

  QueueLockWrite(pQueue,NULL,exit,pPlayerProc,NULL);
  QueueUnlockWrite(pQueue,NULL);
}

==> 1.7.18&file=out_yasapi-1.7.18-src.tar.gz: out_yasapi-1.7.18/libya/ya_player_stub.c <==
/*
 * ya_player_stub.c
 * Copyright (C) 2015-2016 Peter Belkner <pbelkner@snafu.de>
 *
 * This file is part of libya.
 *
 * libya is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * libya is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with libya.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <ya.h>
#include <avrt.h>

static PlayerStub stub;

PlayerStub *PlayerStubCreate(const IPlayer *lpVtbl, Player *pPlayer)
{
  PlayerStub *pStub=&stub;

  /////////////////////////////////////////////////////////////////////////////
  if (pStub->lpVtbl) {
    DMESSAGE("vtbl already in use");
    goto vtbl;
  }

  pStub->lpVtbl=lpVtbl;

  if (pStub->pPlayer) {
    DMESSAGE("player already in use");
    goto player;
  }

  pStub->pPlayer=pPlayer;

#if defined (YA_DEBUG) // {
  /////////////////////////////////////////////////////////////////////////////
  TraceFixIdcs(&trace,lpVtbl->pTraceIdcs);
#endif // }

  /////////////////////////////////////////////////////////////////////////////
  if (QueueCreate(&pStub->queue)<0) {
    DMESSAGE("creating queue");
    goto queue;
  }

  DPUTS(0,"  queue created\n");

  /////////////////////////////////////////////////////////////////////////////
  if (StoreCreate(&pStub->store)<0) {
    DMESSAGE("creating store");
    goto store;
  }

  DPUTS(0,"  store created\n");

  // create the thread ////////////////////////////////////////////////////////
  pStub->hThread=CreateThread(
    NULL,               // _In_opt_   LPSECURITY_ATTRIBUTES lpThreadAttributes,
    0,                  // _In_       SIZE_T dwStackSize,
    PlayerStubThread,   // _In_       LPTHREAD_START_ROUTINE lpStartAddress,
    pStub,              // _In_opt_   LPVOID lpParameter,
    0,                  // _In_       DWORD dwCreationFlags,
    NULL                // _Out_opt_  LPDWORD lpThreadId
  );

  if (NULL==pStub->hThread) {
    DMESSAGE("creating thread");
    goto thread;
  }

  DPUTS(0,"  thread created\n");

  /////////////////////////////////////////////////////////////////////////////
  if (PlayerStubSend(pStub,0,lpVtbl->RunProc)<0) {
    DMESSAGE("running");
    goto run;
  }

  DPUTS(0,"  run request send\n");

  /////////////////////////////////////////////////////////////////////////////
  return pStub;
run:
  CloseHandle(pStub->hThread);
thread:
  StoreDestroy(&pStub->store);
store:
  QueueDestroy(&pStub->queue);
queue:
player:
vtbl:
  return NULL;
}

void PlayerStubDestroy(PlayerStub *pStub)
{
  PlayerStubSend(pStub,1,pStub->lpVtbl->StopProc);
  WaitForSingleObject(pStub->hThread,INFINITE);
  CloseHandle(pStub->hThread);
  StoreDestroy(&pStub->store);
  QueueDestroy(&pStub->queue);
  pStub->pPlayer=NULL;
  pStub->lpVtbl=NULL;
}

const PlayerStub *PlayerStubGet(void)
{
  return &stub;
}

DWORD WINAPI PlayerStubThread(LPVOID lpParameter)
{
  PlayerStub *pStub=lpParameter;
  const IPlayer *lpVtbl=pStub->lpVtbl;
  Player *pPlayer=pStub->pPlayer;
  Queue *pQueue=&pStub->queue;
  PlayerEventProc *EventProc=lpVtbl->EventProc;
  int exit=0;
  DWORD dwTaskIndex=0;
  HANDLE hTask=AvSetMmThreadCharacteristicsW(L"Pro Audio",&dwTaskIndex);
  HANDLE hEvent;
  Request *pRequest;
  int state;
#if defined (YA_THREAD_CO_INITIALIZE) // {
  HRESULT hr;

  hr=CoInitializeEx(
    NULL,                     // _In_opt_ LPVOID pvReserved
	  COINIT_MULTITHREADED		  // _In_     DWORD  dwCoInit
  );

  if (FAILED(hr)) {
    DERROR(S_FALSE,hr);
    DERROR(RPC_E_CHANGED_MODE,hr);
    DMESSAGE("initializing com failed");
    goto coinit;
  }
#endif // }

  while (!exit) {
    hEvent=lpVtbl->GetEvent?lpVtbl->GetEvent(pPlayer):NULL;
    pRequest=QueueLockRead(pQueue,hEvent,EventProc,pPlayer);
    state=pRequest->pPlayerProc(pPlayer,pRequest);
    exit=pRequest->exit;

    if (NULL!=pRequest->pResult)
      pRequest->pResult->state=state;

    QueueUnlockRead(pQueue,pRequest->pResult);
  }

#if defined (YA_THREAD_CO_INITIALIZE) // {
  CoUninitialize();
coinit:
#endif // }

  if (hTask)
    AvRevertMmThreadCharacteristics(hTask);

  return 0;
}

///////////////////////////////////////////////////////////////////////////////
int64_t PlayerStubSendV(PlayerStub *pStub, int exit, PlayerProc *pPlayerProc,
    va_list ap)
{
  Store *pStore=&pStub->store;
  Queue *pQueue=&pStub->queue;
  Result *pResult;
  Request *pRequest;
  int64_t state;

  pResult=StoreGet(pStore);
  pRequest=QueueLockWrite(pQueue,pResult,exit,pPlayerProc,ap);
  QueueUnlockWrite(pQueue,pResult);
    state=pResult->state;
  StorePut(pStore,pResult);

  return state;
}

int64_t PlayerStubSend(PlayerStub *pStub, int exit, PlayerProc *pPlayerProc,
    ...)
{
  va_list ap;
  int64_t state;

  va_start(ap,pPlayerProc);
  state=PlayerStubSendV(pStub,exit,pPlayerProc,ap);
  va_end(ap);

  return state;
}

void PlayerStubPost(PlayerStub *pStub, int exit, PlayerProc *pPlayerProc)
{
  Queue *pQueue=&pStub->queue;

  QueueLockWrite(pQueue,NULL,exit,pPlayerProc,NULL);
  QueueUnlockWrite(pQueue,NULL);
}
